package getoffers;

import java.util.concurrent.ConcurrentLinkedQueue;
import com.fxcore2.O2GLoginRules;
import com.fxcore2.O2GOfferRow;
import com.fxcore2.O2GOffersTableResponseReader;
import com.fxcore2.O2GRequest;
import com.fxcore2.O2GRequestFactory;
import com.fxcore2.O2GResponse;
import com.fxcore2.O2GResponseReaderFactory;
import com.fxcore2.O2GResponseType;
import com.fxcore2.O2GSession;
import com.fxcore2.O2GTable;
import com.fxcore2.O2GTableUpdateType;
import com.fxcore2.O2GTablesUpdatesReader;

public class EventListener {
    private SimpleLog mSimpleLog;
    private ConcurrentLinkedQueue<O2GResponse> mResponseQueue;
    private O2GSession mSession;
    private EventPipe mEventPipe;

    public EventListener(SimpleLog simpleLog, ConcurrentLinkedQueue<O2GResponse> responseQueue, O2GSession session, EventPipe eventPipe) {
        mSimpleLog = simpleLog;
        mResponseQueue = responseQueue;
        mSession = session;
        mEventPipe = eventPipe;
    }

    public void waitNextEvent() {
        boolean bContinue = true;
        while (bContinue) {
            EventType eEventType = mEventPipe.waitEvent();
            if (eEventType == EventType.STATUS) {
                this.onStatusChanged();
            } else if (eEventType == EventType.CONNECTED) {
                this.onConnected();
            } else if (eEventType == EventType.RESPONSE) {
                this.onResponse();
            } else {
                // "How to login" : Logout when you are done.
                mSession.logout();
                bContinue = false;
            }
        }
    }

    private void onResponse() {
        while (true) {
            if (mResponseQueue.size() == 0)
                break;

            O2GResponse response = mResponseQueue.poll();

            if (response.getType() == O2GResponseType.GET_OFFERS || response.getType() == O2GResponseType.TABLES_UPDATES) {
                onOffersRecieved(response);
                if (response.getType() == O2GResponseType.TABLES_UPDATES)
                    onTableUpdateReceive(response);
            } 
        }
    }

    private void onConnected() {
        // Show login rules.
        O2GLoginRules loginRules = mSession.getLoginRules();

        // Has been received offers table snapshot?
        if (loginRules != null && loginRules.isTableLoadedByDefault(O2GTable.OFFERS)) {
            O2GResponse response = loginRules.getTableRefeshResponse(O2GTable.OFFERS);
            onOffersRecieved(response);
        } else
            sendGetOffers();
    }

    private void onOffersRecieved(O2GResponse response) {
        O2GResponseReaderFactory factory = mSession.getResponseReaderFactory();
        if (factory == null)
            return;

        O2GOffersTableResponseReader offersReader = factory.createOffersTableReader(response);
        if (offersReader == null)
            return;

        if (response.getType() == O2GResponseType.TABLES_UPDATES)
            onUpdateOffers(offersReader);
        else
            onOffersTableResponse(offersReader);
    }

    private void onUpdateOffers(O2GOffersTableResponseReader offersReader) {
        int iCount = offersReader.size();

        for (int i = 0; i < iCount; i++) {
            O2GOfferRow offer = offersReader.getRow(i);
            mSimpleLog.log("{0} OfferID={1} Bid={2} Ask={3} Volume={4}", offer.getInstrument(), offer.getOfferID(), offer.getBid(), offer.getAsk(), offer.getVolume());
        }
    }

    private void onOffersTableResponse(O2GOffersTableResponseReader offersReader) {
       // useGenericTableInterface(offersReader);
        useOffersTableIntefaces(offersReader);
    }

    private void useOffersTableIntefaces(O2GOffersTableResponseReader offersReader) {
        mSimpleLog.log("Offers Table");
        for (int i = 0; i < offersReader.size(); i++) {
            O2GOfferRow offer = offersReader.getRow(i);
            printToLogOffer(offer);
        }
    }

    private void sendGetOffers() {
        O2GRequestFactory factory = mSession.getRequestFactory();
        O2GRequest refreshOffers = factory.createRefreshTableRequest(O2GTable.OFFERS);
        mSession.sendRequest(refreshOffers);
    }

     private void onStatusChanged() {
    }


    private void onTableUpdateReceive(O2GResponse response) {
        O2GResponseReaderFactory factory = mSession.getResponseReaderFactory();
        if (factory == null)
            return;

        O2GTablesUpdatesReader updatesReader = factory.createTablesUpdatesReader(response);
        if (updatesReader == null)
            return;

        for (int i = 0; i < updatesReader.size(); i++) {
            O2GTable eTable = updatesReader.getUpdateTable(i);
            if (eTable == O2GTable.TABLE_UNKNOWN)
                continue;

            switch (eTable) {
            case OFFERS: {
                O2GOfferRow offer = updatesReader.getOfferRow(i);
                printToLogOffer(offer);
                break;
            }

            }
        }
    }

    private void printToLogOffer(O2GOfferRow offer) {
        mSimpleLog.log("{0} OfferID={1} Bid={2} Ask={3} Low={4} High={5} Volume={6} QuoteID={7} Time={8} BidTradable={9} AskTradable={10} ContractCurreny={11} Digits={12} PointSize={13} SubscriptionStatus={14} TradingStatus={15} InstrumentType={16} ContractMultiplier={17} ValueDate={18} BuyInterest={19} SellInterest={20}",
                offer.getInstrument(), offer.getOfferID(), offer.getBid(), offer.getAsk(), offer.getLow(), offer.getHigh(), offer.getVolume(), offer.getQuoteID(), offer.getTime().getTime(), offer.getBidTradable(), offer.getAskTradable(), offer.getContractCurrency(), offer.getDigits(), offer.getPointSize(),
                offer.getSubscriptionStatus(), offer.getTradingStatus(), offer.getInstrumentType(), offer.getContractMultiplier(), offer.getValueDate(), offer.getBuyInterest(), offer.getSellInterest());
    }

}